/*
 * LBProxyThread.java
 *
 * Created on September 7, 2005, 11:08 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package org.msblabs.http;

import java.net.*; 
import java.io.*; 
import java.util.*;
import java.net.URLDecoder;

public class LBProxy { 

    //keep pending the urls pending approval for up to 5 minutes
    private static final int flushOldURLs = 5 * 60;

    //----------- Public variables our Threads will access
    public static final String lbHeader = "X-Scanned-By: LineBreaker Anti-Phishing Proxy - www.msblabs.org";
    public static String [] xssSignatures;
    
    //----------- Private stuff for me
    private int port;
    
    private ArrayList<PendingEntry> pendingConfirms;

    
    
    
//-------------------------------------------------------- Constructor    
    
    public LBProxy (int port) { 
        this.port = port;
        xssSignatures = LBProxy.loadFile("/datafiles/bad-params.txt");
        pendingConfirms = new ArrayList<PendingEntry>();
    }     
    
//----------------------------------------- MISC, SHARED FUNCTIONS
    
    
    /**
     * Sanitizes input, convert < > ", etc into entities
     * @param s String to sanatize
     * @return String cleaned up String
     */
    public static String sanitize(String s) {
        StringBuilder sb = new StringBuilder();
        for(char c: s.toCharArray()) {
            if(c == '\"') {
                sb.append("&quot;");
            } else if (c == '&') {
                sb.append("&amp;");
            } else if (c == '<') {
                sb.append("&lt;");
            } else if (c == '>') {
                sb.append("&gt;");
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }
    
    private static String wrapTo(String s, int col) {
        StringBuilder sb = new StringBuilder();
        int i=0;
        for(char c : s.toCharArray()) {
            sb.append(c);

            if(++i > col) {
                sb.append("<BR>\n");
                i = 0 ;
            }
        }
        return sb.toString();
    }
    
    public static byte [] generateImage(String fn) {

        InputStream fromFile = LBProxy.class.getResourceAsStream(fn);
        //Read in the entire response from the server
        try {
            int count =0;
            byte [] buffer = new byte[4096];
            MyByteArray mba = new MyByteArray();
            while ((count = fromFile.read(buffer)) != -1) {
                mba.add(buffer, count);
            }
            return mba.getBytes();
        } catch (Exception e) {
            //silently fail
        }
        return null;
    }
    
    public static String generateWarningPage(String url, String xssString)  throws UnsupportedEncodingException {
        
        StringBuilder body = new StringBuilder();
        body.append("<HTML>\n");
        body.append("<center>\n");
        body.append("<h1>Warning! Possible Phishing Attack Detected!</h1><BR>\n");
        body.append("<h3>Line Breaker has detected that the URL</h3>\n<p><i>");
        body.append(wrapTo(sanitize(URLDecoder.decode(url, "UTF-8")), 65));
        body.append("</i>\n<p><h3>Contains a suspicious parameter</h3>\n<p><b>\"");
        body.append(xssString);
        body.append("\"</b><BR><BR>If you still want to go to this page, <a href=\"");
        body.append(url);
        body.append("\">Click Here</a><BR><BR>\n<img src=\"http://this-is-not-a-real-url-it-is-for-line-braker.com/line-breaker-logo.png\">\n");
        body.append("<BR><BR><img src=\"http://this-is-not-a-real-url-it-is-for-line-braker.com/spi-logo.png\">");
        body.append("</center>");
        body.append("</HTML>");

        StringBuilder sb = new StringBuilder();
        sb.append("HTTP/1.1 200 OK");
        sb.append("\r\n");
        sb.append("Connection: close");
        sb.append("\r\n");        
        sb.append("Content-type: text/html");
        sb.append("\r\n");
        sb.append("Content-Length: ");
        sb.append(body.toString().length());
        sb.append("\r\n");
        sb.append("\r\n");
        sb.append(body.toString());
        return sb.toString();
    }
    
    
    public static String [] loadFile(String fn) {
        
        ArrayList<String> ext = new ArrayList<String>();
        
        BufferedReader in = null;

        in = new BufferedReader(
                new InputStreamReader(
                LBProxy.class.getResourceAsStream(fn)));

        String inputLine=null;
        
        try {
            while ((inputLine = in.readLine()) != null) {
                if(inputLine.indexOf('#') < 0 && inputLine.length() > 0) {
                    ext.add(inputLine);
                }
            }
        } catch (IOException e) {

        }
        if(ext.size() > 0) {
            return (String[]) ext.toArray(new String[0]);
        }
        return null;
    }

    
    public void listen() {
        ServerSocket listen = null;
        try {
            listen = new ServerSocket(port);
            System.out.println("LineBreaker is listening on port " + port);
            while (true) {
                Socket incoming = listen.accept();      
                LBProxyThread thread = new LBProxyThread(incoming, this);
                thread.start();
           }  
       } catch (Exception e) {
           System.out.println("Generic Proxy Error");
           e.printStackTrace();
           System.exit(1);
       } finally  {
           try {
               listen.close();
           } catch (Exception e2 ) {
            //do nothing,. we are dying}
           }
       }                
    }
//====================================================== Confirming Functions
    
    public synchronized void addPendingURL(String url) {
        pendingConfirms.add( new PendingEntry(url));
        //cleaning can be expensive, so only do it when we add
        cleanOldPendingURLs();
    }
    
    
    public boolean removePendingURL(String url) {
        //search each one
        for(PendingEntry e: pendingConfirms) {
            if(e.hashEquals(url)) {
                //we are holding this
                //remove it
                pendingConfirms.remove(e);
                return true;
            }
        }
        return false;
    }
    
    /** clear out old pending URL confirms */
    public synchronized void cleanOldPendingURLs() {
        long now = (new Date()).getTime();
        boolean reprocess;
        do {
            reprocess = false;
            for(PendingEntry e : pendingConfirms) {
                //older than 5 minutes?
                if(e.created + (flushOldURLs * 1000) < now) {
                    pendingConfirms.remove(e);
                    reprocess = true;
                    break;
                }
            }
            
        } while(reprocess);
        
    }
    
    
//====================================================================== MAIN   
    
    
    public static void main(String [] args) {
        LBProxy lbp;

        if(args.length > 0) {
            lbp = new LBProxy(Integer.parseInt(args[0]));
        } else {
            lbp = new LBProxy(8080);
        }
        lbp.listen();
    }

    
} 